@@ -36,7 +36,6 @@ import pycountry
 import sys
 import gzip
 import shutil
-from geograpy.prefixtree import PrefixTree
 from geograpy.wikidata import Wikidata
 from lodstorage.sql import SQLDB
 from geograpy.utils import remove_non_ascii
@@ -107,6 +106,22 @@ class Region(object):
         '''
         region=Region()
         region.name=record['regionName']
+        region.iso="%s-%s" % (record['countryIsoCode'],record['regionIsoCode']) 
+        return region   
+    
+    @staticmethod
+    def fromWikidata(record):
+        '''
+        create  a region from a Wikidata record
+        
+        Args:
+            record(dict): the records as returned from a Query
+            
+        Returns:
+            Region: the corresponding region information
+        '''
+        region=Region()
+        region.name=record['regionLabel']
         region.iso=record['regionIsoCode'] 
         return region   
     
@@ -167,6 +182,8 @@ class Locator(object):
         self.db_file = db_file or self.db_path+"/locs.db"
         self.view="GeoLite2CityLookup"
         self.sqlDB=SQLDB(self.db_file,errorDebug=True)
+        self.getAliases()
+        self.dbVersion="2020-09-27 16:48:09"
         
     @staticmethod
     def resetInstance():
@@ -186,12 +203,13 @@ class Locator(object):
             Locator.locator=Locator(correctMisspelling=correctMisspelling,debug=debug)
         return Locator.locator
         
-    def locate(self,places):
+    def locateCity(self,places):
         '''
-        locate a city, region country combination based on the places information
+        locate a city, region country combination based on the given wordtoken information
         
         Args:
-            places(list): a list of place tokens e.g. "Vienna, Austria"
+            places(list): a list of places derived by splitting a locality e.g.  "San Francisco, CA"
+            leads to "San Francisco", "CA"
         
         Returns:
             City: a city with country and region details
@@ -201,46 +219,21 @@ class Locator(object):
         country=None
         cities=[]
         regions=[]
-        level=1
-        prefix=''
+        # loop over all word elements
         for place in places:
-            isPrefix=self.isPrefix(prefix+place,level)
-            isAmbigous=False
-            if not isPrefix:
-                prefix=''
-            checkPlace=prefix+place
-            if isPrefix:
-                isAmbigous=self.isAmbiguousPrefix(prefix+place)
-                level+=1
-                prefix="%s%s " % (prefix,place)
-            if not isPrefix or isAmbigous:
-                foundCountry=self.getCountry(checkPlace)
-                if foundCountry is not None:
-                    country=foundCountry
-                foundCities=self.cities_for_name(checkPlace)
-                cities.extend(foundCities)
-                foundRegions=self.regions_for_name(checkPlace)
-                regions.extend(foundRegions)
+            place=place.strip()
+            if place in self.aliases:
+                place=self.aliases[place]
+            foundCountry=self.getCountry(place)
+            if foundCountry is not None:
+                country=foundCountry
+            foundCities=self.cities_for_name(place)
+            cities.extend(foundCities)
+            foundRegions=self.regions_for_name(place)
+            regions.extend(foundRegions)
         foundCity=self.disambiguate(country, regions, cities)
         return foundCity
     
-    def isAmbiguousPrefix(self,name):
-        '''
-        check if the given name is an ambiguous prefix
-        
-        Args:
-            name(string): the city name to check
-            
-        Returns:
-            bool: True if this is a known prefix that is ambigous that is there is also a city with 
-            such a name    
-        '''
-        query="select name from ambiguous where name=?"
-        params=(name,)
-        aResult=self.sqlDB.query(query,params)
-        result=len(aResult)>0
-        return result
-    
     def isISO(self,s):
         '''
         check if the given string is an ISO code
@@ -248,26 +241,9 @@ class Locator(object):
         Returns:
             bool: True if the string is an ISO Code
         '''
-        m=re.search(r"^[0-9A-Z]{1,3}$",s)
+        m=re.search(r"^([A-Z]{1,2}\-)?[0-9A-Z]{1,3}$",s)
         result=m is not None
         return result
-
-    def isPrefix(self,name,level):
-        '''
-        check if the given name is a city prefix at the given level
-        
-        Args:
-            name(string): the city name to check
-            level(int): the level on which to check (number of words)
-            
-        Returns:
-            bool: True if this is a known prefix of multiple cities e.g. "San", "New", "Los"
-        '''
-        query="SELECT count from prefixes where prefix=? and level=?"
-        params=(name,level)
-        prefixResult=self.sqlDB.query(query,params)
-        result=len(prefixResult)>0
-        return result
                
     def disambiguate(self,country,regions,cities,byPopulation=True): 
         '''
@@ -283,8 +259,8 @@ class Locator(object):
         '''
         if self.debug:
             print("countries: %s " % country)
-            print("regions: %s" % regions)
-            print("cities: %s" % cities)
+            print("regions: %s" % "\n\t".join(str(r) for r in regions))
+            print("cities: %s" % "\n\t".join(str(c) for c in cities))
         foundCity=None
         # is the city information unique?
         if len(cities)==1:
@@ -339,13 +315,16 @@ class Locator(object):
         Returns:
             list: the list of cities for this region
         '''
-        regions=[]
+        regions=[]    
         if self.isISO(region_name):
-            regionRecords=self.places_by_name(region_name,'regionIsoCode')
+            columnName="regionIsoCode"
         else:
-            regionRecords=self.places_by_name(region_name, 'regionName')
+            columnName='regionLabel'
+        query="SELECT * from regions WHERE %s = (?)" % (columnName)
+        params=(region_name,)
+        regionRecords=self.sqlDB.query(query,params)
         for regionRecord in regionRecords:
-            regions.append(Region.fromGeoLite2(regionRecord))
+            regions.append(Region.fromWikidata(regionRecord))
         return regions                     
     
     def correct_country_misspelling(self, name):
@@ -394,6 +373,12 @@ class Locator(object):
         country=None
         if pcountry is not None:
             country=Country.fromPyCountry(pcountry)
+        #if country is None:
+        #    query="SELECT * FROM countries WHERE countryLabel = (?)"""
+        #    params=(name,)
+        #    countryRecords=self.sqlDB.query(query,params)
+        #    if len(countryRecords)>0:
+        #        pass
         return country
     
     def getView(self):
@@ -428,15 +413,19 @@ class Locator(object):
         Returns:
             list: a list of Geolite2 City-Locator dicts
         '''
-        cities=[]
+        cities=self.readCSV("GeoLite2-City-Locations-en.csv")
+        return cities
+    
+    def readCSV(self,fileName):
+        records=[]
         cur_dir = os.path.dirname(os.path.realpath(__file__))
-        csvfile=cur_dir + "/data/GeoLite2-City-Locations-en.csv"
+        csvfile="%s/data/%s" % (cur_dir,fileName)
         with open(csvfile) as info:
             reader = csv.DictReader(info)
             for row in reader:
-                cities.append(row)
-        return cities
-    
+                records.append(row)
+        return records
+     
     def recreateDatabase(self):
         '''
         recreate my lookup database
@@ -457,8 +446,7 @@ class Locator(object):
             self.populateFromWikidata(self.sqlDB)
             self.getWikidataCityPopulation(self.sqlDB)
             self.createViews(self.sqlDB)
-            self.populate_PrefixTree(self.sqlDB,self.getView())
-            self.populate_PrefixAmbiguities(self.sqlDB,self.getView())
+            self.populate_Version(self.sqlDB)
     
         elif not hasData:
             url="http://wiki.bitplan.com/images/confident/locs.db.gz"
@@ -472,10 +460,32 @@ class Locator(object):
         if not os.path.isfile(self.db_file):
             raise("could not create lookup database %s" % self.db_file)
             
-            
+    def populate_Version(self,sqlDB):
+        '''
+        populate the version table
+        
+        Args:
+            sqlDB(SQLDB): target SQL database
+        '''
+        versionList=[{"version":self.dbVersion}]
+        entityInfo=sqlDB.createTable(versionList,"Version","version",withDrop=True)
+        sqlDB.store(versionList,entityInfo)
+        
+    def getAliases(self):
+        '''
+        get the aliases hashTable
+        '''
+        aliases=self.readCSV("aliases.csv")
+        self.aliases={}
+        for alias in aliases:
+            self.aliases[alias['name']]=alias['alias']
+        
     def populateFromWikidata(self,sqlDB):
         '''
         populate countries and regions from Wikidata
+        
+        Args:
+            sqlDB(SQLDB): target SQL database
         '''
         self.populate_Countries(sqlDB)
         self.populate_Regions(sqlDB)
@@ -504,16 +514,22 @@ FROM City_wikidata
     def populate_Countries(self,sqlDB):
         '''
         populate database with countries from wikiData
+        
+        Args:
+            sqlDB(SQLDB): target SQL database
         '''
         print("retrieving Country data from wikidata ... (this might take a few seconds)")
         wikidata=Wikidata()
         wikidata.getCountries()
-        entityInfo=sqlDB.createTable(wikidata.countryList[:5000],"countries","countryIsoCode",withDrop=True)
-        sqlDB.store(wikidata.countryList,entityInfo)
+        entityInfo=sqlDB.createTable(wikidata.countryList,"countries",None,withDrop=True,sampleRecordCount=200)
+        sqlDB.store(wikidata.countryList,entityInfo,fixNone=True)
 
     def populate_Regions(self,sqlDB):
         '''
         populate database with regions from wikiData
+        
+        Args:
+            sqlDB(SQLDB): target SQL database
         '''
         print("retrieving Region data from wikidata ... (this might take a minute)")
         wikidata=Wikidata()
@@ -642,46 +658,6 @@ FROM citiesWithPopulation
         for viewDDL in viewDDLs:
             sqlDB.execute(viewDDL)
         
-    def populate_PrefixAmbiguities(self,sqlDB,view):
-        '''
-        create a table with ambiguous prefixes
-        
-        Args:
-            sqlDB(SQLDB): the SQL database to use
-            view(str): the view to use
-        '''
-        query="""SELECT distinct name 
-from %s c join prefixes p on c.name=p.prefix
-order by name""" % view
-        ambigousPrefixes=sqlDB.query(query)
-        entityInfo=sqlDB.createTable(ambigousPrefixes, "ambiguous","name",withDrop=True)
-        sqlDB.store(ambigousPrefixes,entityInfo)
-        return ambigousPrefixes
-        
-    def populate_PrefixTree(self,sqlDB,view):
-        '''
-        calculate the PrefixTree info
-        
-        Args:
-            sqlDb(SQLDB): the SQL Database to use
-            view(string): the view to use
-        
-        Returns:
-            PrefixTree: the prefix tree
-        '''
-        query="""select 
-  distinct name from GeoLite2CityLookup 
-  where not name=""
-union 
-  select distinct wikidataName as name 
-  from GeoLite2CityLookup %s""" % view
-        nameRecords=sqlDB.query(query)
-        trie=PrefixTree()   
-        for nameRecord in nameRecords:
-            name=nameRecord['name']
-            trie.add(name)
-        trie.store(sqlDB)   
-        return trie     
     
     def db_recordCount(self,tableList,tableName):
         '''
@@ -717,8 +693,14 @@ union
         hasCities=self.db_recordCount(tableList,"citiesWithPopulation")>10000
         hasCountries=self.db_recordCount(tableList,"countries")>100
         hasRegions=self.db_recordCount(tableList,"regions")>1000
+        hasVersion=self.db_recordCount(tableList,"Version")==1
+        versionOk=False
+        if hasVersion:
+            query="SELECT version from Version"
+            dbVersionList=self.sqlDB.query(query)
+            versionOk=dbVersionList[0]['version']==self.dbVersion
         #hasWikidataCities=self.db_recordCount(tableList,'City_wikidata')>100000
-        ok=hasCities and hasRegions and hasCountries
+        ok=hasVersion and versionOk and hasCities and hasRegions and hasCountries
         return ok
     
 __version__ = '0.1.15'
